# -*- coding: utf-8 -*-
"""
Create, Read, Update, Delete functionality

"""

from sqlalchemy.orm import Session
from fastapi.responses import JSONResponse
from . import models
from .schemas import PersonCreate, OrganisationCreate


def get_organisation(db: Session, organisation_id: int):
    return db.query(models.Organisation).filter(models.Organisation.id == organisation_id).first()


def get_all_organisations(db: Session, skip : int = 0, limit: int = None):
    return db.query(models.Organisation).offset(skip).limit(limit).all()

def get_organisation_by_name(db: Session, name: str):
    return db.query(models.Organisation).filter(models.Organisation.name == name).first()


def create_organisation(db: Session, organisation: OrganisationCreate):
    db_organisation = models.Organisation(**organisation.dict())
    db.add(db_organisation)
    result = db.commit()
    db.refresh(db_organisation)
    return db_organisation


def get_person(db: Session, person_id: int):
    return db.query(models.Person).filter(models.Person.id == person_id).first()


def get_all_persons(db: Session, skip : int = 0, limit: int = None):
    return db.query(models.Person).offset(skip).limit(limit).all()

def get_person_by_name(db: Session, name: str):
    return db.query(models.Person).filter(models.Person.name == name).first()


def create_person(db: Session, person: PersonCreate):
    db_person = models.Person(**person.dict())
    db.add(db_person)
    result = db.commit()
    db.refresh(db_person)
    return db_person
