# coding: utf-8
"""
class Person (Base)
===================
"""
from sqlalchemy import Boolean, Column, Integer, String, text
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata

class Person(Base):
    """ Table "public.persons"

    +-----------+------------------------+-----------+----------+-------------------------------------+
    |  Column   |          Type          | Collation | Nullable |               Default               |
    +===========+========================+===========+==========+=====================================+
    | id        | integer                |           | not null | nextval('persons_id_seq'::regclass) |
    +-----------+------------------------+-----------+----------+-------------------------------------+
    | name      | character varying(64)  |           | not null |                                     |
    +-----------+------------------------+-----------+----------+-------------------------------------+
    | email     | character varying(128) |           | not null |                                     |
    +-----------+------------------------+-----------+----------+-------------------------------------+
    | phone     | character varying(32)  |           | not null |                                     |
    +-----------+------------------------+-----------+----------+-------------------------------------+
    | isprivate | boolean                |           | not null |                                     |
    +-----------+------------------------+-----------+----------+-------------------------------------+
    Indexes:
        "persons_pkey" PRIMARY KEY, btree (id)
    Referenced by:
        TABLE "station_roles" CONSTRAINT "station_roles_person_id_3bd9c160_fk_persons_id" FOREIGN KEY (person_id) REFERENCES persons(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "timeseries_roles" CONSTRAINT "timeseries_roles_person_id_3e26200e_fk_persons_id" FOREIGN KEY (person_id) REFERENCES persons(id) DEFERRABLE INITIALLY DEFERRED
    """
    __tablename__ = 'persons'

    id = Column(Integer, primary_key=True, server_default=text("nextval('persons_id_seq'::regclass)"))
    name = Column(String(64), nullable=False)
    email = Column(String(128), nullable=False)
    phone = Column(String(32), nullable=False)
    isprivate = Column(Boolean, nullable=False)

